import * as argparse from 'argparse';
export { argparse };

export interface ArgumentParserNodeInterface {
	name?: string;
	readonly subparsers: argparse.SubParser;
	readonly level: number;
	readonly pathArray: string[];
	readonly path: string;
	get(name: string, options?: argparse.ArgumentParserOptions): ArgumentParserNodeInterface;
}

export type ArgumentParserNodeOptions =
	{ options: argparse.ArgumentParserOptions, defaults: argparse.ArgumentParserOptions, subparserOptions?: argparse.SubparserOptions } & ( { parent: ArgumentParserNode, name: string } | {} );

export class ArgumentParserNode implements ArgumentParserNodeInterface {
	name?: string;
	parent?: ArgumentParserNode;
	root?: ArgumentParserNode;
	defaultOptions: argparse.ArgumentParserOptions;
	subparserOptions: argparse.SubparserOptions;
	parser: argparse.ArgumentParser;
	children: {[key: string]: ArgumentParserNode} = {};
	_subparsers: argparse.SubParser|undefined = undefined;
	constructor(options: ArgumentParserNodeOptions) {
		if ( 'parent' in options ) {
			this.name = options.name;
			this.parent = options.parent;
			this.defaultOptions = this.parent.defaultOptions;
			this.subparserOptions = options.subparserOptions || this.parent.subparserOptions || {};
			this.parser = this.parent.subparsers.addParser(options.name, options.options || this.defaultOptions );
		}
		else {
			this.defaultOptions = options.defaults || {};
			this.subparserOptions = options.subparserOptions || {};
			this.parser = new argparse.ArgumentParser(options.options || this.defaultOptions);
		}
	}
	get(name: string, options?: argparse.ArgumentParserOptions): ArgumentParserNode {
		if ( !this.children.hasOwnProperty(name) ) {
			this.children[name] = new (this.constructor as any)({ parent: this, name, options });
		}
		return this.children[name];
	}
	get subparsers(): argparse.SubParser {
		if ( !this._subparsers ) {
			const options: any = Object.assign({}, this.subparserOptions);
			options.dest = `subparser_${this.level}`;
			this._subparsers = this.parser.addSubparsers(options);
			//(this.parser as any)._subparsers.defaultValue = null
		}
		return this._subparsers;
	}
	get level(): number {
		if ( this.parent ) {
			return this.parent.level + 1;
		}
		else return 0;
	}
	get pathArray(): string[] {
		if ( this.parent ) {
			const tmp = this.parent.pathArray;
			tmp.push(this.name as string);
			return tmp;
		}
		else return [];
	}
	get path(): string {
		return this.pathArray.join('/');
	}
}

export type ArgumentParserTree = ArgumentParserNode;
export const ArgumentParserTree = ArgumentParserNode;

export default ArgumentParserTree;
